(*
 *
 *                     Test program of the S25R system.
 *                     Copyright (C) 2005  Fumio KAWAMATA
 *                       Created date: October 23, 2005
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.
 *
 *)
unit TestS25R_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, IdStackWindows, IdGlobal, Buttons, Menus,
  ComCtrls;


type
  TFormMain = class(TForm)
    PanelTop: TPanel;
    LabeledEditDNSServer: TLabeledEdit;
    BitBtnClose: TBitBtn;
    StatusBar1: TStatusBar;
    MainMenu: TMainMenu;
    MainMenuFile: TMenuItem;
    MainMenuOption: TMenuItem;
    MainMenuHelp: TMenuItem;
    BitBtnCheckS25R: TBitBtn;
    MainMenuHelpVersion: TMenuItem;
    MainMenuOptionFont: TMenuItem;
    FontDialog: TFontDialog;
    MainMenuFileCheckS25R: TMenuItem;
    MainMenuFileExit: TMenuItem;
    PageControl: TPageControl;
    Header: TTabSheet;
    Result: TTabSheet;
    MemoHeader: TMemo;
    MemoResult: TMemo;
    MemoRegExp: TMemo;
    LabelRegExp: TLabel;
    MainMenuEdit: TMenuItem;
    MainMenuEditClearHeader: TMenuItem;
    MainMenuOptionSeparator: TMenuItem;
    MainMenuOptionIniFileName: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MainMenuHelpVersionClick(Sender: TObject);
    procedure MainMenuOptionFontClick(Sender: TObject);
    procedure BitBtnCloseClick(Sender: TObject);
    procedure MainMenuFileExitClick(Sender: TObject);
    procedure PanelTopResize(Sender: TObject);
    procedure MainMenuEditClearHeaderClick(Sender: TObject);
    procedure MainMenuFileCheckS25RClick(Sender: TObject);
  private
    procedure InitializeValues;
    function ReadParametersFromIniFile(IniFileName: string): boolean;
    function WriteParametersToIniFile(IniFileName: string): boolean;
  public
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}
uses
  VerInfo, FontInfo, FormStatus,
  BRegExp,
  (* BregExp.pas  Copyright (C) Osamu Takeuchi All rights Reserved. *)
  (* BregExp.pas includes Japanese. *)
  (* BregExp.pas requires BregExp.dll, Copyright (C) Tatsuo Baba *)
  IniFiles,
  S25R ;

const
  RegExpOfIPAddress='m/[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/';
  CRLF = chr($0D)+chr($0A);

var
  IniFileFullPath: String;
  WorkStrings: TStrings;



(* *****************************************************************************
 * ArrangeHeaders
 *   converts headers to Japanese,
 *   combine header lines which consist of several rows (ex: "Received:"
 *   header) into one line.
 * ****************************************************************************
 *)
procedure ArrangeHeaders(HeaderList: TStrings);
var
  Count:           Integer;
  FirstChar:       char;
  I:               Integer;
  LastField:       string;
  NumberOfHeaders: Integer;
  UpperStr:        string;
  TmpStr:          string;
begin
  NumberOfHeaders:=HeaderList.Count;
  LastField:='';
  I:=0; Count:=0;
  repeat
    TmpStr:=HeaderList[I];
    UpperStr:=UpperCase(TmpStr);
    FirstChar:=TmpStr[1];
    if (FirstChar=' ')or(FirstChar=char($09)) then
    begin
      if LastField='SUBJECT:' then
        HeaderList[I-1]:=HeaderList[I-1]+Trim(TmpStr)
      else
        HeaderList[I-1]:=HeaderList[I-1]+' '+Trim(TmpStr);
      HeaderList.Delete(I);
    end
    else
    begin
      HeaderList[I]:=TmpStr;
      if pos('SUBJECT:', UpperStr)=1 then
        LastField:='SUBJECT:'
      else
        LastField:='';
      inc(I);
    end;
    inc(Count);
  until Count>=NumberOfHeaders;
end;




(* *****************************************************************************
 * Get command line options
 * *****************************************************************************
 *)
procedure GetCommandLineOptions;
var
  I: Integer;
  S: String;
begin
  if ParamCount=0 then
    Exit;
  for I:=1 to ParamCount do
  begin
    S:=ParamStr(I);
    if Pos('-I', S)=1 then
    begin
      Delete(S, 1, 2);
      IniFileFullPath:=S;
    end
  end;
end;



(* *****************************************************************************
 * Initialize values
 * *****************************************************************************
 *)
procedure TFormMain.InitializeValues;
begin
  IniFileFullPath:=ChangeFileExt(ParamStr(0), '.ini');
  LabeledEditDNSServer.Text:='Enter IP address of the DNS server here.';
  MemoRegExp.Lines.Clear;
  MemoRegExp.Lines.Add('Enter match string here.  An example is described in below.');
  MemoRegExp.Lines.Add('m/[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+\]\) by sbmx[0-9]+/i');
  MemoHeader.Lines.Clear;
  MemoHeader.Lines.Add('Paste header block here.');
  MemoResult.Lines.Clear;
  MemoResult.Lines.Add('Result will be displayed here.');
end;



(* *****************************************************************************
 * Read parameters from INI file
 * *****************************************************************************
 *)
function TFormMain.ReadParametersFromIniFile(IniFileName: string): boolean;
var
  Count: Integer;
  IniFile: TIniFile;
  I: Integer;
  S: String;
begin
  ReadParametersFromIniFile:=True;
  IniFile := TIniFile.Create(IniFileName);
  try
    S:=IniFile.ReadString('General', 'MatchString', '');
    if S<>'' then
    begin
      MemoRegExp.Lines.Clear;
      MemoRegExp.Lines.Add(S);
    end;
    S:=IniFile.ReadString('General', 'DNSServer', '');
    if S<>'' then
      LabeledEditDNSServer.Text:=S;
    Count:=IniFile.ReadInteger('Memo', 'Count', 0);
    if Count>0 then
    begin
      MemoHeader.Lines.Clear;
      for I:=0 to Count -1 do
      begin
        MemoHeader.Lines.Add(IniFile.ReadString('Memo', IntToStr(I), ''));
      end;
    end;
  except
    ReadParametersFromIniFile:=False;
  end;
  IniFile.Free;
end;



(* *****************************************************************************
 * Write parameters to INI file
 * *****************************************************************************
 *)
function TFormMain.WriteParametersToIniFile(IniFileName: string): boolean;
var
  I: Integer;
  IniFile: TIniFile;
  S: String;
begin
  WriteParametersToIniFile:=True;
  IniFile := TIniFile.Create(IniFileName);
  S:='';
  if MemoRegExp.Lines.Count>0 then
  begin
    for I:=0 to MemoRegExp.Lines.Count-1 do
    begin
      S:=S+MemoRegExp.Lines[I];
    end;
  end;
  try
    IniFile.WriteString('General', 'MatchString', S);
    IniFile.WriteString('General', 'DNSServer', LabeledEditDNSServer.Text);
    IniFile.WriteInteger('Memo', 'Count', MemoHeader.Lines.Count);
    if MemoHeader.Lines.Count>0 then
    begin
      for I:=0 to MemoHeader.Lines.Count-1 do
      begin
        IniFile.WriteString('Memo', IntToStr(I), '"'+MemoHeader.Lines[I]+'"');
      end;
    end;
  except
    WriteParametersToIniFile:=False;
  end;
  IniFile.Free;
end;



(* *****************************************************************************
 * FormCreate event handler
 * *****************************************************************************
 *)
procedure TFormMain.FormCreate(Sender: TObject);
var
  AppIni: TIniFile;
  S25RRulePath: String;
begin
  WorkStrings:=TStringList.Create;
  InitializeValues;
  GetCommandLineOptions;
  if FileExists(IniFileFullPath)=False then
  begin
    AppIni:=TIniFile.Create(IniFileFullPath);
    AppIni.UpdateFile;
    AppIni.Free;
    //MessageDlg('The program created "'+IniFileFullPath+'" file.',mtInformation,[mbOK],0);
    FormMain.Position:=poScreenCenter;
  end;
  FormStatus.ReadFormStatusFromIniFile(FormMain, IniFileFullPath);
  FormMain.Caption:=FormMain.Caption+'  '+GetVersionInfo('FileVersion');
  FormMain.MainMenuOptionIniFileName.Caption:='INI file name: '+IniFileFullPath;
  ReadParametersFromIniFile(IniFileFullPath);
  PageControl.ActivePageIndex:=0;
  MemoHeader.Refresh;
  FontInfo.ReadFontInfoFromIniFile(FormMain, IniFileFullPath);
  S25RRulePath:=ExtractFilePath(ParamStr(0));
  S25RRulePath:=S25RRulePath+'S25R_Rules.txt';
  S25R.CreateThem;
  S25R.LoadRules(S25RRulePath);
end;



(* *****************************************************************************
 * FormDestroy event handler
 * *****************************************************************************
 *)
procedure TFormMain.FormDestroy(Sender: TObject);
begin
  S25R.FreeThem;
  FormStatus.WriteFormStatusToIniFile(FormMain, IniFileFullPath);
  WriteParametersToIniFile(IniFileFullPath);
  FontInfo.WriteFontInfoToIniFile(FormMain, IniFileFullPath);
  WorkStrings.Free;
end;



(* *****************************************************************************
 * Version display
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuHelpVersionClick(Sender: TObject);
var
  S: String;
begin
  S:='Portions of this software are Copyright (c) 1993 - 2005, Chad Z. Hower (Kudzu) and the Indy Pit Crew - http://www.nevrona.com/Indy/'+CRLF;
  S:=S+'PzConv.pas Copyright (C) 1998 Pz factory'+CRLF;
  S:=S+'BREGEXP.DLL Copyright (C) Tatsuo Baba'+CRLF;
  S:=S+'BregExp.pas Copyright (C) Osamu Takeuchi'+CRLF;
  S:=S+'The S25R System is an idea by ASAMI Hideo, Japan';
  VerInfo.DisplayVersionInfo(S);
end;



(* *****************************************************************************
 * Font selection
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuOptionFontClick(Sender: TObject);
var
  I: Integer;
begin
  FontDialog.Font:=Font;
  if FontDialog.Execute=True then
  begin
    Font:= FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
    begin
      Screen.Forms[I].Font:=Font;
    end;
  end;
end;



(* *****************************************************************************
 * Close button click event handler
 * *****************************************************************************
 *)
procedure TFormMain.BitBtnCloseClick(Sender: TObject);
begin
  Close;
end;



(* *****************************************************************************
 * Menu [File|Exit] menu item selection event handler
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuFileExitClick(Sender: TObject);
begin
  Close;
end;


procedure TFormMain.PanelTopResize(Sender: TObject);
begin
  MemoRegExp.Width:=PanelTop.Width-16;
  LabeledEditDNSServer.Width:=PanelTop.Width-16;
end;



procedure TFormMain.MainMenuEditClearHeaderClick(Sender: TObject);
begin
  MemoHeader.Lines.Clear;
  MemoResult.Lines.Clear;
  PageControl.ActivePageIndex:=0;
end;

procedure TFormMain.MainMenuFileCheckS25RClick(Sender: TObject);
var
  DNSServer: String;
  IPAddress2: String;
  HostName: String;
  I: Integer;
  IPAddress: String;
  Matched: Boolean;
  MatchString: String;
  S: String;
  S25RResult: Integer;
begin
  if MemoHeader.Lines.Count=0 then
  begin
    MessageDlg('Header is empty.', mtWarning, [mbOK], 0);
    Exit;
  end;

  if MemoRegExp.Lines.Count=0 then
  begin
    MessageDlg('Match string is empty.', mtWarning, [mbOK], 0);
    Exit;
  end;

  if LabeledEditDNSServer.Text='' then
  begin
    MessageDlg('DNS Server is empty.', mtWarning, [mbOK], 0);
    Exit;
  end;

  MatchString:='';
  for I:=0 to MemoRegExp.Lines.Count-1 do
  begin
    MatchString:=MatchString+MemoRegExp.Lines[I];
  end;

  WorkStrings.Clear;
  for I:=0 to MemoHeader.Lines.Count-1 do
  begin
    WorkStrings.Add(MemoHeader.Lines[I]);
  end;
  (* I thought above 5 lines are same as WorkStrings:=MemoHeader.Lines;,
   * However, if I do so following ArrangeHeaders() does not work well
   *)

  (* Setp 1: Arrange Headers and display arranged header in the MemoResult *)
  MemoResult.Lines.Clear;
  MemoResult.Lines.Add(''); (* Dummy row.  This row will be fill later *)
  MemoResult.Lines.Add(''); (* Line space *)
  MemoResult.Lines.Add('*** Step 1: Arrange Headers ***');
  ArrangeHeaders(WorkStrings);
  if WorkStrings.Count>0 then
  begin
    for I:=0 to WorkStrings.Count-1 do
    begin
      MemoResult.Lines.Add(WorkStrings[I]);
    end;
  end;

  (* Step 2: Extract Received field which includes sender's IP address. *)
  MemoResult.Lines.Add(''); (* Line space *)
  MemoResult.Lines.Add('*** Step 2: Extract sender''s IP address ***');
  I:=WorkStrings.Count-1;
  Matched:=False;
  while (I>=0)and(Matched=False) do
  begin
    S:=WorkStrings[I];
    if brx.Match('m/^Received:/i', S)=True then
    begin
      if brx.Match(MatchString, WorkStrings[I])=True then
      begin
        Matched:=True;
      end;
    end;
    if Matched=False then
      dec(I);
  end;
  if Matched=False then
  begin
    MemoResult.Lines.Add('Could not find Received header which includes sender''s IP address.');
    Exit;
  end;
  MemoResult.Lines.Add('Target Received header is in below.');
  MemoResult.Lines.Add(S);
  S:=brx[0];
  if brx.Match(RegExpOfIPAddress, S)=False then
  begin
    MemoResult.Lines.Add('Could not extract IP address from '+S);
    Exit;
  end;
  IPAddress:=brx[0];
  MemoResult.Lines.Add('Extract IP address is '+IPAddress);

  (* Step 3: Check by S25R system *)
  MemoResult.Lines.Add(''); (* Line space *)
  MemoResult.Lines.Add('*** Step3: S25R Check ***');
  DNSServer:=LabeledEditDNSServer.Text;
  S25R.CheckRulesByIPAddress(DNSServer, IPAddress, HostName, IPAddress2, S25RResult);
  MemoResult.Lines.Add('Hostname: '+HostName);
  MemoResult.Lines.Add('Forward lookuped IP address: '+IPAddress2);
  case S25RResult of
     0: S:='Clearn';
    -1: S:='S25R Reverse lookup failure';
    -2: S:='S25R Forward lookup failure';
    -3: S:='S25R Paranoid check failure';
    else
        S:='Matched S25R Rule '+IntToStr(S25RResult);
  end;
  MemoResult.Lines.Add('Result: '+S);
  MemoResult.Lines[0]:=S;
  PageControl.ActivePageIndex:=1;
end;


end.
