(*
 *   Copyright (C) 2005 Fumio Kawamata
 *   Created date: October 15, 2005
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.
 *)


unit lookup;

interface


uses
  Classes, IdComponent, IdBaseComponent, IdTCPClient, IdTCPConnection, IdStack, IdGlobal, IdDNSResolver;

var
  AResolver: TIdDNSResolver;
  TCPClient: TIdTCPClient;

  function IPtoHostName(IPAddress: String): String;
  (* Converts IP address to Host name
   * Returns Host name
   *)

  function HostNameToIPAddress(HostName: String): String;
  (* Converts Host name to IP address
   * Returns IP address
   *)

  procedure HostNameToIPAddresses(DNSServer: String; HostName: String; IPAddresses: TStrings);

  procedure CreateComponents;

  procedure FreeComponents;


implementation

uses
  SysUtils, Dialogs;


function IPtoHostName(IPAddress: String): String;
var
  HostName: String;
begin
  try
    HostName:=GStack.HostByAddress(IPAddress, ID_DEFAULT_IP_VERSION);
    Result:=HostName;
  except
    Result:='';
  end;
end;


function HostNameToIPAddress(HostName: String): String;
var
  IPAddress: String;
begin
  try
    IPAddress:=GStack.ResolveHost(HostName, ID_DEFAULT_IP_VERSION);
    Result:=IPAddress;
  except
    Result:='';
  end;
end;


procedure CreateComponents;
begin
  AResolver:=TIdDNSResolver.Create;
  TCPClient:=TIdTCPClient.Create;
end;


procedure FreeComponents;
begin
  AResolver.Free;
  TCPClient.Free;
end;

procedure  HostNameToIPAddresses(DNSServer: String; HostName: String; IPAddresses: TStrings);
var
  Count: Integer;
  I: Integer;
begin
  IPAddresses.Clear;
  AResolver.Host:=DNSServer;
  AResolver.QueryType:=[qtA];
  try
    AResolver.Resolve(HostName);
    Count:=AResolver.QueryResult.Count;
  except
    Count:=0;
  end;
  if Count=0 then exit;
  for I:=0 to Count-1 do
  begin
    if AResolver.QueryResult.Items[I] is TARecord then
      with TARecord(AResolver.QueryResult[I]) do
      begin
        IPAddresses.Add(IPAddress);
      end;
  end;
end;



end.
