(* ****************************************************************************
 *                     Version Information Utility
 *                  Copyright (C) 1998-2004 Fumio Kawamata
 *
 *  You can get Version Information of the program by using this unit.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 *  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.  You must be have it.
 * ****************************************************************************
 *)

unit VerInfo;

interface

procedure DisplayVersionInfo(AdditionalMessage: String);
function GetVersionInfo(ItemStr:String):String;

implementation

uses
  Windows, Forms, Dialogs ;

function GetVersionInfo(ItemStr:String):String;
(* ****************************************************************************
 * ItemStr must be in 'ProductName','FileDescription','FileVersion',
 * 'LegalCopyright','CompanyName','InternalName','OriginalFilename',
 * 'ProductVersion','Comment','LegalTradeMarks',etc.
 * Example:
 *   GetVersionInfo('FileVersion') returns '1.1.0.5'.
 * ****************************************************************************
 *)
const
  Language     = '0411'; (* Japanese *)
  CharacterSet = '03A4'; (* Windows, Japanse(Shift JIS, X-0208) *)
var
  FullInfoSize    : DWORD;
  ItemInfoSize    : DWORD;
  pFullInfo       : Pointer;
  pItemInfo       : Pointer;
  Ret             : DWORD;
begin
  FullInfoSize:=GetFileVersionInfoSize(pChar(Application.ExeName),Ret);
  if FullInfoSize>0 then begin
    GetMem(pFullInfo,FullInfoSize);
    try
      GetFileVersionInfo(PChar(Application.ExeName),0,FullInfoSize,pFullInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet
        +'\'+ItemStr),pItemInfo,ItemInfoSize);
      Result:=PChar(pItemInfo);
    finally
      FreeMem(pFullInfo,FullInfoSize);
    end;
  end
  else Result:='';
end;

procedure DisplayVersionInfo;
const
  CRLF = chr($0D)+chr($0A);
var
  Mess : String;
begin
  Mess:=VerInfo.GetVersionInfo('ProductName')+'    Original filename: '
       +VerInfo.GetVersionInfo('OriginalFilename')+CRLF
       +VerInfo.GetVersionInfo('FileDescription')+CRLF
       +'Version '+VerInfo.GetVersionInfo('FileVersion')+CRLF
       +VerInfo.GetVersionInfo('LegalCopyright')+CRLF
       +'This product is a free software distributed under the GNU GPL version 2'+CRLF
       +'Homepage: http://openlab.jp/fumio/';
  MessageDlg(Mess+CRLF+CRLF+AdditionalMessage,mtInformation,[mbOK],0);
end;


end.
